%union {
	int intval;
	double floatval;
	char *strval;
	int subtok;
}

%token NAME
%token INTNUM
%token STRING

%left OR
%left AND
%left NOT
%left <subtok> COMPARISON
%left '+' '-'
%left '*' '/'
%nonassoc UMINUS

%token SELECT
%token UNION
%token JOIN
%token INTERSECT
%token DIVIDE
%token PROJECT
%token COMPARISON
%token DIFFERENCE
%token PRODUCT

%%

expression:
			algebra ';'
		{
			printf("Expression resolved.\n");
		}
	;

algebra:	
			relation
		|	select_statement
		|	project_statement
		|	join_statement
		|	union_statement
		|	intersection_statement
		|	difference_statement
		|	product_statement
		;

select_statement:	
			SELECT '(' algebra ')' condition
	;

project_statement:
			PROJECT '(' algebra ')' '(' attribute_comma_list ')'
	;

join_statement:
			JOIN '(' algebra ')' '(' algebra ')' condition
	;

union_statement:
			UNION '(' algebra ')' '(' algebra ')'
	;

intersection_statement:
			INTERSECT '(' algebra ')' '(' algebra ')'
	;

difference_statement:
			DIFFERENCE '(' algebra ')' '(' algebra ')'
	;

product_statement:
			PRODUCT '(' algebra ')' '(' algebra ')'
	;
			
condition:
		'(' condition_statement ')'
	;

condition_statement: 
	|	comparison_predicate OR comparison_predicate
	|	comparison_predicate AND comparison_predicate
	|	NOT comparison_predicate
	|	comparison_predicate
	;

comparison_predicate:
		atom
	|	attribute_ref
	|	condition
	|	predicate
	;

predicate:
		comparison_predicate COMPARISON comparison_predicate
	;

relation:	
		NAME 
	|	NAME '.' NAME
	;

attribute_comma_list:
		attribute_ref
	|	attribute_ref ',' attribute_comma_list
	;


attribute_ref:
		NAME
	|	NAME '.' NAME 
	|	NAME '.' NAME '.' NAME
	;

atom:
		STRING
	|	INTNUM
	;
%%
